// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009,2013 Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009,2013 Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#include <assignment_stack.hpp>
#include <constraint_connectivity_info.hpp>
#include <constraint_aged_info.hpp>

using namespace std;
using namespace hcsp;

//virtual
void constraint_aged_info::put_in_queue(queue_type)
{
    throw domain_error("Can't put_in_queue for constraint_aged_info");
}

// virtual
const iset &constraint_aged_info::domain(unsigned param_index) const
{
    return m_assignment_stack.var_domain_after_modification(
        to_variable(param_index),
        m_after_modification);
}

//virtual
void constraint_aged_info::set_domain(unsigned, iset)
{
    throw domain_error("Domain can't be updated for a constraint_aged_info");
}

//virtual
unsigned constraint_aged_info::arity() const
{
    return m_const_base_info.arity();
}

void constraint_aged_info::deallocate_watch(
    unsigned param_index, watch_type type, watch_handle_t watch_handle)
{
    if (m_p_base_info)
        m_p_base_info->deallocate_watch(param_index, type, watch_handle);
    else
        throw domain_error("Can't deallocate watches on const info objects");
}

watch_handle_t constraint_aged_info::allocate_watch(
    unsigned param_index, watch_type type)
{
    if (m_p_base_info)
        return m_p_base_info->allocate_watch(param_index, type);

    throw domain_error("Can't allocate watches on const info objects");
}

void constraint_aged_info::deallocate_value_watch(
    unsigned param_index, iset::value_type value, watch_handle_t watch_handle)
{
    if (m_p_base_info)
        m_p_base_info->deallocate_value_watch(param_index, value, watch_handle);
    else
        throw domain_error("Can't deallocate watches on const info objects");
}

watch_handle_t constraint_aged_info::allocate_value_watch(
    unsigned param_index, iset::value_type value)
{
    if (m_p_base_info)
        return m_p_base_info->allocate_value_watch(param_index, value);
    throw domain_error("Can't allocate watches on const info objects");
}


// virtual
int constraint_aged_info::latest_modified_param(unsigned except) const
{
    if (!m_allow_latest_modification_query)
        return -1;
    int best_param=-1;
    change_index_t latest_modification;
    latest_modification= m_assignment_stack.NO_PREVIOUS_MODIFICATION;
    for (unsigned i=0 ; i != m_const_base_info.arity() ; ++i) {
        if (i == except)
            continue;
        change_index_t mod= m_assignment_stack.var_modification_entry_before(
            to_variable(i), m_after_modification+1);
        if ( m_assignment_stack.is_earlier_modification(latest_modification,
                                                        mod) ) {
            latest_modification= mod;
            best_param= i;
        }
    }
    return best_param;
}

// virtual
void constraint_aged_info::print(std::ostream & out) const throw()
{
    for (unsigned i=0 ; i!= arity() ; ++i) {
        if (i!=0)
            out << ", ";
        print_param(out, i);
        out << "={" << domain(i) << "}";
    }
}

//virtual
void constraint_aged_info::print_prefix(std::ostream & out) const throw()
{
    m_const_base_info.print_prefix(out);
}

//virtual
void constraint_aged_info::print_param(
    std::ostream &out, unsigned param_index) const throw()
{
    m_const_base_info.print_param(out, param_index);
}


