// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009,2014, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009,2014, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#include <iostream>
#include <solver.hpp>
#include <constraint_connectivity_info.hpp>
#include <constraint_connectivity_info.inl>

using namespace hcsp;
using namespace std;

constraint_connectivity_info::~constraint_connectivity_info()
{}


constraint_connectivity_info & 
constraint_connectivity_info::operator<<(unsigned variable_index)
{
    m_arguments.push_back(variable_index);
    return *this;
}

bool
constraint_connectivity_info::reduced_between_activations(unsigned param_index)
    const
{
    unsigned var_index= m_arguments.at(param_index);
    return
        m_p_solver->var_modification_time(var_index) > m_last_activation_time;
}

//virtual
constraint_connectivity_info::queue_type
constraint_connectivity_info::originating_queue() const
{
    return m_p_solver->current_queue();
}

//virtual
void constraint_connectivity_info::put_in_queue(queue_type queue)
{
    m_p_solver->put_in_queue(m_constraint_index, queue);
}


void constraint_connectivity_info::finalize_prev_activation(
    time_type current_time)
{
    m_last_activation_time= current_time;
    if ( !m_last_activation_reduced )
        return;
    for (unsigned i=0 ; i != m_arguments.size() ; ++i) {
        if ( reduced_by_last_activation(i) )
            m_p_solver->notify_modification(to_variable(i),m_constraint_index);
    }
}

const iset & constraint_connectivity_info::domain(unsigned param_index) const
{
    unsigned var_index= m_arguments.at(param_index);
    return m_p_solver->var_domain(var_index);
}

void constraint_connectivity_info::set_domain(unsigned param_index,
                                              iset new_domain)
{
    unsigned var_index= m_arguments.at(param_index);
    if (new_domain.size() == m_p_solver->var_domain(var_index).size() &&
        new_domain.empty() == m_p_solver->var_domain(var_index).empty())
        return;
    m_last_activation_reduced= true;
    return m_p_solver->set_var_domain(var_index, move(new_domain),
                                      m_constraint_index);
}

// virtual
void constraint_connectivity_info::deallocate_watch(
    unsigned param_index, watch_type type, watch_handle_t watch_handle)
{
    m_p_solver->deallocate_watch(to_variable(param_index), type, watch_handle);
}

//virtual
watch_handle_t constraint_connectivity_info::allocate_watch(
    unsigned param_index, watch_type type)
{
    unsigned var_index= m_arguments.at(param_index);
    return m_p_solver->allocate_watch(var_index, type, m_constraint_index);
}

// virtual
void constraint_connectivity_info::deallocate_value_watch(
    unsigned param_index, iset::value_type value, watch_handle_t watch_handle)
{
    m_p_solver->deallocate_value_watch(to_variable(param_index), value,
                                       watch_handle);
}

//virtual
watch_handle_t constraint_connectivity_info::allocate_value_watch(
    unsigned param_index, iset::value_type value)
{
    unsigned var_index= m_arguments.at(param_index);
    return m_p_solver->allocate_value_watch(var_index, value,
                                            m_constraint_index);
}

void constraint_connectivity_info::print_prefix(ostream & out) const throw()
{
    out << "constraint[" << m_constraint_index << "]";
} 
 
void constraint_connectivity_info::print(ostream & out) const throw()
{
    print_prefix(out);
    out << " (";
    for (unsigned i=0 ; i != m_arguments.size() ; ++i) {
        if ( i!=0 )
            out << ",";
        print_param(out, i);
    }
    out << ")";
} 
 
void constraint_connectivity_info::print_param(ostream & out,
                                               unsigned param) const throw()
{ 
    out << m_p_solver->var_printer(to_variable(param));
}
