// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2014, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2014, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */

#include <constraint_explicit_info.hpp>


#include <iostream>
#include <variable_print_strategy.hpp>

using namespace hcsp;
using namespace std;

constraint_explicit_info::constraint_explicit_info(unsigned in_arity)
    : m_domains(in_arity)
{}

constraint_explicit_info::constraint_explicit_info(
    const constraint_explicit_info &) = default;

constraint_explicit_info& constraint_explicit_info::operator=(
    const constraint_explicit_info &) = default;

// virtual
constraint_explicit_info::~constraint_explicit_info() {}

void constraint_explicit_info::set_var_printer(unsigned arg,
                                               const variable_printer & printer)
{
    if (m_var_printers.size() <= arg)
        m_var_printers.resize(arg+1);
    m_var_printers[arg]= printer;
}

// virtual
void constraint_explicit_info::print(std::ostream &out) const throw()
{
    print_prefix(out);
    out << " (";
    for (unsigned i=0 ; i != m_domains.size() ; ++i) {
        if ( i!=0 )
            out << ", ";
        print_param(out, i);
        if (m_domains[i].singleton())
            out << "=" << m_domains[i];
        else if (m_domains[i].empty())
            out << "={}";
        else if (m_domains[i].everything())
            out << "={*}";
        else if (m_domains[i].min() == iset::MIN_DOMAIN &&
                 ((iset::size_type)m_domains[i].max()-m_domains[i].min() ==
                  m_domains[i].size()-1))
            out << "<=" << m_domains[i].max();
        else if (m_domains[i].max() == iset::MAX_DOMAIN &&
                 ((iset::size_type)m_domains[i].max()-m_domains[i].min() ==
                  m_domains[i].size()-1))
            out << ">=" << m_domains[i].min();
        else
            out << "={" << m_domains[i] << "}";
    }
    out << ")";
}

// virtual
void constraint_explicit_info::print_prefix(ostream & out) const throw()
{
    out << "constraint";
} 
 

// virtual
void constraint_explicit_info::print_param(std::ostream &out,
                                           unsigned param) const throw()
{
    if (param < m_var_printers.size() && m_var_printers[param].is_valid())
        out << m_var_printers[param];
    else
        out << "P" << param;
}
