// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009,2014 Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009,2014 Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#include <cstdlib>
#include <iostream>
#include <constraint_runtime_info.hpp>

using namespace std;
using namespace hcsp;

// At least one virtual or used method of the class must be put into this cpp,
// otherwise show_self may be optimized-out.
//virtual
void constraint_runtime_info::print(std::ostream & ) const throw()
{
}

void constraint_runtime_info::show_self() const
{
    print(cout);
    cout << "\n";
    for (unsigned i=0 ; i != arity() ; ++i)
        cout << i << ": " << domain(i) << "\n";
    cout.flush();
}

unsigned constraint_runtime_info::accurate_size_bound() const
{
    static bool is_init = false;
    unsigned value= 10;
    if (!is_init) {
        is_init= true;
        //! \todo Should be handled by hcsp::flags rather than by getenv
        const char *str= getenv("ACCURATE_SIZE_BOUND");
        if (str)
            value= strtoul(str, NULL, 0);
    }
    static const unsigned ret= value;
    return ret;
}

unsigned constraint_runtime_info::accurate_size_pair_bound() const
{
    static bool is_init = false;
    unsigned value= 50;
    if (!is_init) {
        is_init= true;
        //! \todo Should be handled by hcsp::flags rather than by getenv
        const char *str= getenv("ACCURATE_SIZE_PAIR_BOUND");
        if (str)
            value= strtoul(str, NULL, 0);
    }
    static const unsigned ret= value;
    return ret;
}


unsigned constraint_runtime_info::max_accurate_ranges() const
{
    static bool is_init = false;
    unsigned value= 20;
    if (!is_init) {
        is_init= true;
        //! \todo Should be handled by hcsp::flags rather than by getenv
        const char *str= getenv("MAX_ACCURATE_RANGES");
        if (str)
            value= strtoul(str, NULL, 0);
    }
    static const unsigned ret= value;
    return ret;
}
