// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */

#include <utilities.hpp>
#include <equivalence_classes.hpp>

using namespace std;
using namespace hcsp;


void equivalence_classes::unite(unsigned element1, unsigned element2)
{
    unsigned equiv1= find(element1);
    unsigned equiv2= find(element2);
    if (equiv1 == equiv2)
        return;
    // Only the subsumed element has to change its pointer to the
    // representative. By choosing the subsumed to be the smallest of the
    // pair (equiv1,equiv2) we are required only to allocate enough to
    // hold 'subsumed', but not required to hold 'representative', which is
    // bigger, because 'representative' does not have to point anywhere.
    unsigned representative= max(equiv1, equiv2);
    unsigned subsumed= min(equiv1, equiv2);
    if (subsumed >= initialized_elements())
        m_parents.resize(subsumed+1, NO_ELEMENT);
    parent(subsumed, representative);
}

unsigned equivalence_classes::find(unsigned element) const
{
    // First find the representative of the equivalence set. This value
    // will be returned at the end of the method.
    unsigned representative= element;
    while ( has_parent(representative))
        representative= parent(representative);

    // Next, fix all elements on the path from 'element' until before
    // 'representative' to point to 'representative'.
    // This compaction phase is the main cause for the efficiency of the
    // union-find data structure.
    unsigned current= element;
    while ( current != representative) {
        unsigned next= parent(current);
        parent(current, representative);
        current= next;
    }
    return representative;
}
