// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2010, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2010, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#include <formula.hpp>
#include "formula_impl.hpp"

using namespace std;

namespace hcsp {

// explicit
formula::formula(bool value)
{
    if (value)
        link( true_formula_impl::get() );
    else
        link( false_formula_impl::get() );
}

formula::formula(unsigned variable, const iset & set)
{
    construct(literal_data(variable, set));
}

void formula::construct(const literal_data &literal)
{
    if (literal.m_set.empty())
        link(false_formula_impl::get());
    else if (literal.m_set.everything())
        link(true_formula_impl::get());
    else {
        clause_data clause;
        clause.reserve(1);
        clause.push_back(literal);
        link(new clause_formula_impl(clause));
    }
}

formula::formula(const clause_data &clause)
{
    if (clause.empty())
        link(false_formula_impl::get());
    else
        link(new clause_formula_impl(clause));
}

void formula::constrain_solver(solver & solver) const
{
    m_formula->constrain_solver(solver);
}

bool formula::evaluate(const solver & solver) const
{
    return m_formula->evaluate(solver);
}

//static
void formula::mark_linked(formula_impl & formula)
{
    // formula.link() is called indirectly from within this method in order
    // to avoid #include <formula_impl.hpp> in formula.hpp 
    formula.link();
}

//static
void formula::mark_unlinked(formula_impl & formula)
{
    // formula.unlink() is called indirectly from within this method in order
    // to avoid #include <formula_impl.hpp> in formula.hpp 
    formula.unlink();
}

formula operator!(const formula &a)
{
    return formula(a.m_formula->make_not());
}

formula operator&&(const formula &a, const formula &b)
{
    return formula(a.m_formula->and_with(b.m_formula));
}

formula operator||(const formula &a, const formula &b)
{
    return formula(a.m_formula->or_with(b.m_formula));
}

void formula::print(std::ostream &out, const variable_print_strategy &strategy)
    const
{
    if (m_formula)
        m_formula->print(out, strategy);
    else
        out << "NULL";
}


} // namespace hcsp
