// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2014 Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2014 Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#include <int128.hpp>

#include <stdexcept>
#include <cstdint>

#include <utilities.hpp>
using namespace std;
using namespace hcsp;

// static
inline uint64_t int128::divide_u64(uint64_t numerator, uint64_t denominator,
                                   rounding_mode mode)
{
    if (mode == ROUND_UP)
        return hcsp::ceil_div(numerator, denominator);
    else
        return numerator/denominator;
}

// static
inline int64_t int128::divide_s64(int64_t numerator, int64_t denominator,
                                  rounding_mode mode)
{
    if (mode == ROUND_TO_ZERO)
        return numerator/denominator;
    else if (mode == ROUND_UP)
        return hcsp::ceil_div(numerator, denominator);
    else 
        return hcsp::floor_div(numerator, denominator);
}

int128 int128::divide(int32_t in, rounding_mode mode) const
{
    bool this_is_short= (m_val[1]+ (m_val[0] >> 63)) == 0;
    bool is_neg= (in < 0) != is_negative();
    if (this_is_short && !is_neg)
        return int128(divide_s64(m_val[0], in, mode));
    int128 abs_this= is_negative() ? -*this :  *this;
    int64_t abs_in= in >=0 ? int64_t(in) : int64_t(-1-in)+1;

    int128 ret;
    rounding_mode abs_mode= mode;
    if (is_neg)
        abs_mode= mode == ROUND_DOWN ? ROUND_UP : ROUND_DOWN;
    if (this_is_short) {
        HCSP_ASSERT(is_neg);
        ret.m_val[0]= divide_u64(abs_this.m_val[0], abs_in, abs_mode);
        return -ret;
    }
    ret.m_val[1]= int64_t(abs_this.m_val[1])/abs_in;
    int32_t remainder = int64_t(abs_this.m_val[1])%abs_in;
    int64_t mid = (int64_t(remainder) << 32) | (abs_this.m_val[0] >> 32);
    remainder = mid % abs_in;
    ret.m_val[0] = (mid/abs_in) << 32;
    int64_t rest = ((abs_this.m_val[0] & 0xffffffff)+
                    (int64_t(remainder) << 32));
    ret.m_val[0] += divide_s64(rest, abs_in, abs_mode);
    return is_neg ? -ret : ret;
}
