// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2011, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2011, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#include <iostream>
#include <solver.hpp>
#include <opb_constraint.hpp>
#include <opb_expression.hpp>
#include <opb_context.hpp>

using namespace std;
using namespace hcsp;
using namespace hcsp::opb;


constraint::~constraint()
{
}

bool constraint::apply(context & context) const
{
    if (context.trace_construction())
        cout << "[" << *this << "].apply()\n";
    if (tautology())
        return true;
    if (falsity()) {
        if (context.trace_failure())
            cout << "constraint is trivially false: " << *this;
        return false;
    }
    
    expression::constraint_type bound;
    if (m_direction == EQ) {
        bound= expression::CONSTRAIN_EQUAL;
    } else {
        HCSP_ASSERT(m_direction == GE);
        bound= expression::CONSTRAIN_AT_LEAST;
        if (m_value == 1 && m_expression->all_coefficients_positive()) {
            m_expression->add_or_between_conjunctions(context);
            return true;
        }
    }
    unsigned variable= m_expression->get_evaluating_variable(context, bound);
    
    const iset & domain = context.get_solver().var_domain(variable);
    if (!domain.contains(m_value)) {
        if (context.trace_failure())
            cout << "can't satisfy constraint: " << *this;
        return false;
    }
    context.get_solver().set_var_domain(variable, iset(m_value));
    return true;
}

void constraint::normalize(context & context)
{
    if (context.trace_construction())
        cout << "[" << *this << "].normalize()\n";
    m_expression->normalize(context, m_value);
    if (m_expression->empty()) {
        if (m_direction == EQ) {
            if (m_value==0)
                m_tautology= true;
            else
                m_falsity= true;
        } else {
            HCSP_ASSERT(m_direction == GE);
            if (m_value <= 0 )
                m_tautology= true;
            else
                m_falsity= true;
        }
    }
    if (context.trace_construction())
        cout << "normalize()-->[" << *this << "]\n";
}

namespace hcsp { namespace opb {
std::ostream& operator<<(std::ostream&out, const constraint &data)
{
    out << *data.m_expression << data.m_direction << data.m_value;
    if (data.falsity())
        out << ", i.e., false";
    else if (data.tautology())
        out << ", i.e., true";
    return out << ";";
}

std::ostream& operator<<(std::ostream&out, constraint::direction direction)
{
    switch (direction) {
      case constraint::GE: out << ">="; break;
      case constraint::EQ: out << "="; break;
    }
    return out;
}
} } // namespace hcsp::opb
