// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2010, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2010, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */

#include <proof_checker_visitor.hpp>

using namespace std;
using namespace hcsp;

//virtual
void proof_checker_visitor::visit_resolution(
    unsigned pivot,
    int resolvent1_id, const clause_data & resolvent1,
    int resolvent2_id, const clause_data & resolvent2,
    int consequent_id, const clause_data & consequent)
{
    visit_clause(resolvent1_id, resolvent1, true);
    visit_clause(resolvent2_id, resolvent2, true);
    visit_clause(consequent_id, consequent, false);
    clause_data literals;
    literals.reserve(resolvent1.size()+resolvent2.size());

    iset pivot_set;
    for (unsigned i=0 ; i != resolvent1.size() ; ++i)
        if (resolvent1[i].m_variable == pivot)
            pivot_set= resolvent1[i].m_set;
        else
            literals.push_back(resolvent1[i]);
    
    for (unsigned i=0 ; i != resolvent2.size() ; ++i)
        if (resolvent2[i].m_variable == pivot)
            pivot_set.intersect(resolvent2[i].m_set);
        else
            literals.push_back(resolvent2[i]);

    literals.push_back(literal_data(pivot,pivot_set));

    if ( !consequent.equivalent(literals) ) {
        m_is_correct= false;
        if (m_p_report)
            *m_p_report << "Resolution between clause[" << resolvent1_id
                        << "] and clause[" << resolvent2_id
                        << "] gave wrong clause[" << consequent_id << "]\n";
    }
}

//virtual
void proof_checker_visitor::visit_initial_resolution(
    unsigned variable, const iset & domain,
    int resolvent_id, const clause_data & resolvent,
    int consequent_id, const clause_data & consequent)
{
    visit_clause(resolvent_id, resolvent, true);
    visit_clause(consequent_id, consequent, false);
    clause_data resolved= resolvent;
    for (unsigned i= resolved.size() ; i > 0 ; ) {
        --i;
        if (resolved[i].m_variable == variable) {
            resolved[i].m_set.intersect(domain);
            bool domain_error= !resolved[i].m_set.empty();
            if (domain_error)
                m_is_correct= false;
            if (domain_error && m_p_report)
                *m_p_report << "Resolution of clause[" << resolvent_id
                            << "] with initial domain of "
                            << var_printer(variable)
                            << " does not empty the domain\n";
            resolved[i].swap(resolved.back());
            resolved.pop_back();
        }
    }
    bool different= !resolved.equivalent(consequent);
    if (different)
        m_is_correct= false;
    if (different  && m_p_report)
        *m_p_report << "Resolution of clause[" << resolvent_id
                    << "] with initial domain of " << var_printer(variable)
                    << " is different than passed clause[" << consequent_id
                    << "]\n";
}

void proof_checker_visitor::visit_clause(
    int clause_id, const clause_data & clause, bool is_old)
{
    std::map<int, clause_data>::iterator it= m_clauses.find(clause_id);
    if ( it == m_clauses.end() ) {
        if ( m_collector.is_explanation_clause(clause_id))
            is_old= true;
        if (is_old) {
            clause_data got;
            m_collector.get_clause(clause_id, got);
            bool error= !got.equivalent(clause);
            if (error)
                m_is_correct= false;
            if (error && m_p_report)
                *m_p_report << "clause[" << clause_id << "] "
                            << "has different literals than passed by "
                            << "the proof algorithm\n";
        } 
        m_clauses[clause_id]= clause;
    } else {
        bool error= !clause.equivalent(it->second);
        if (error) 
            m_is_correct= false;
        if (error && m_p_report)
            *m_p_report << "clause[" << clause_id << "] "
                        << "has different literals than previously\n";
    }
}

