// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2012, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2012, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#include <cmath>
#include <cstdlib>
#include <cstring>
#include <unistd.h>
#include <typeinfo>
#include <iostream>
#include <sstream>
#include <iomanip>

#include <timer.hpp>
#include <utilities.hpp>

using namespace std;
using namespace hcsp;

class timer::impl
{
  public:
    impl() : m_microseconds( user_time_microseconds() ) {}

    void print_cpu_diff(std::ostream & out, const impl & earlier_time) const;
  private:
    //! Print time differences. Can be used for CPU/real/system times.
    void print_diff(std::ostream & out, microseconds_t earlier,
                    microseconds_t now) const;
    
    microseconds_t m_microseconds;
}; // class timer::impl

void timer::impl::print_cpu_diff(std::ostream &out, const impl &earlier) const
{
    print_diff(out, earlier.m_microseconds, m_microseconds);
}

void timer::impl::print_diff(std::ostream & out, microseconds_t earlier,
                             microseconds_t now) const
{
    microseconds_t microseconds= now-earlier;
    microseconds+=500; // round up, such that 0.0006 seconds becomes 0.001
    int milliseconds = microseconds/1000;
    int seconds= milliseconds/1000;
    milliseconds -= seconds*1000;
    int minutes= seconds/60;
    seconds -= minutes*60;
    int hours= minutes/60;
    minutes -= hours*60;
    ios_state_restore restorer(out);
    out << setfill('0') << setw(0);
    if (hours)
        out << hours << ':' << setw(2);
    if (minutes || hours)
        out << minutes << ':' << setw(2);
    out << seconds << '.' << setw(3) << milliseconds;
}

timer & timer::operator=(const timer& other)
{
    *m_impl= *other.m_impl;
    return *this;
}

timer::~timer()
{
    delete m_impl;
    m_impl= NULL;
}

void timer::construct()
{
    HCSP_ASSERT(m_impl == NULL);
    m_impl= new impl;
}

void timer::print_cpu_diff(std::ostream & out, const timer & earlier_time) const
{
    m_impl->print_cpu_diff(out, *earlier_time.m_impl);
}

string timer::cpu_diff_str(const timer & earlier_time) const
{
    ostringstream out;
    print_cpu_diff(out, earlier_time);
    return out.str();
}
