// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009,2014, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009,2014, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#include <var_watchers.hpp>
#include <constraint_connectivity_info.hpp>

using namespace std;

namespace hcsp {
var_watchers_base::watch_list_t watch_handle_t::null_watch_list;
namespace {
template <class T>
bool is_in_list(const T& container, unsigned constraint_index)
{
    for (const auto & elem : container)
        if ( elem.m_constraint_index == constraint_index )
            return true;
    return false;
}
}

template <class Set>
var_watchers_impl<Set>::for_each_cleaner::~for_each_cleaner()
{
    if ( !m_p_watches )
        return;                 // It is a nested for_each.
    HCSP_ASSERT( m_p_watches->m_in_for_each );
    m_p_watches->m_in_for_each= false;
    
    var_watchers_impl & watchers= *const_cast<var_watchers_impl*>(m_p_watches);
    
    for (unsigned i= watchers.m_key_watchers.size() ; i > 0 ; --i) {
        bool is_empty= watchers.m_key_watchers[i-1].m_watchers.empty();
        if ( is_empty ) {
            watchers.m_key_watchers[i-1]= move(watchers.m_key_watchers.back());
            watchers.m_key_watchers.pop_back();
        }
    }
}

template <class Set>
watch_handle_t var_watchers_impl<Set>::allocate_watch(iset::value_type key,
                                                      unsigned constraint_index)
{
    HCSP_ASSERT( ! is_key_watched_by_constraint( key, constraint_index ) );
    key_watchers *p_key_watchers= find_key_watchers(key);

    if ( !p_key_watchers) {
        m_key_watchers.resize(m_key_watchers.size()+1);
        p_key_watchers= &m_key_watchers.back();
        p_key_watchers->m_key= key;
    }
    watcher_info_type watcher={constraint_index};
    ++m_watch_count;
    if (m_is_fifo)
    {
        p_key_watchers->m_watchers.push_back(watcher);
        return prev(p_key_watchers->m_watchers.end());
    }
    else
    {
        p_key_watchers->m_watchers.push_front(watcher);
        return p_key_watchers->m_watchers.begin();
    }
}

template <class Set>
void var_watchers_impl<Set>::deallocate_watch(
    iset::value_type key, watch_handle_t watch_handle)
{
    HCSP_PROFILE_CURRENT;

    key_watchers *p_key_watchers= find_key_watchers(key);
    if (!p_key_watchers)
        throw out_of_range("Can't deallocate nonexistent watch");

    if ( watch_handle.empty()) 
        throw out_of_range("Can't deallocate nonexistent watch");
    
    auto it = watch_handle.get_it();
    p_key_watchers->m_watchers.erase(it);
    --m_watch_count;
    
    if ( p_key_watchers->m_watchers.empty() ) {
        if ( !m_in_for_each ) {
            *p_key_watchers= move(m_key_watchers.back());
            p_key_watchers= NULL;
            m_key_watchers.pop_back();
        }
    }
}

template <class Set>
bool var_watchers_impl<Set>::is_key_watched_by_constraint(
    iset::value_type key, unsigned constraint_index) const
{
    const key_watchers *p_key_watchers= find_key_watchers( key );
    if ( !p_key_watchers )
        return false;
    return is_in_list(p_key_watchers->m_watchers, constraint_index);
}

template <class Set>
const typename var_watchers_impl<Set>::key_watchers *
var_watchers_impl<Set>::find_key_watchers(iset::value_type key) const
{
    for (unsigned i=0 ; i != m_key_watchers.size() ; ++i) {
        if ( m_key_watchers[i].m_key == key )
            return &m_key_watchers[i];
    }
    return NULL;
}

template class var_watchers_impl<iset>;
template class var_watchers_impl<small_iset>;
}  // namespacd hcsp
