// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2012,2014, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2012,2014, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#ifndef HCSP_FZN_SOLVER_HPP_
#define HCSP_FZN_SOLVER_HPP_

#include <string>
#include <iosfwd>
#include <table_propagators.hpp>
#include <solver.hpp>
#include <parser_context.hpp>

namespace hcsp {
class hcsp_flags;
class fzn_solver : public solver {
  public:
    fzn_solver(std::istream & in,
               const std::string & file_name,
               const hcsp_flags & flags,                
               table_propagator_base::memory_manager& manager);
    virtual iset::value_type evaluate_objective() const
    {
        return m_context.evaluate_objective();
    }
    virtual void print_solution(std::ostream & out) const;
    //! Run solve() and print the results in fzn format
    void solve_and_print(std::ostream & out);
  private:
    //! Register the constraints of FlatZinc into the parser_context.
    void register_constraint_types();
    parser_context m_context;
}; // class fzn_solver
} // namespace hcsp


#endif // HCSP_FZN_SOLVER_HPP_
