// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#ifndef HCSP_ISET_HPP_
#define HCSP_ISET_HPP_

/**
 * @def USE_BIG_ISET
 * If defined, the solver and the associated will be compiled with
 * big_iset as the parent of iset. If undefined then the parent of iset is
 * small_iset.
 */
#ifdef USE_BIG_ISET
#    include <big_iset.hpp>
#    define PARENT_OF_ISET big_iset
#    define HCSP_SET_MODEL big_model
#else
#    include <small_iset.hpp>
#    define PARENT_OF_ISET small_iset
#    define HCSP_SET_MODEL small_model
#endif

namespace hcsp { namespace HCSP_SET_MODEL {

//! A class efficiently representing a set of integers.
class iset : public PARENT_OF_ISET {
    typedef PARENT_OF_ISET parent_type;
  public:
    //! Create an iset with an empty domain
    iset() { }
    iset(const value_range_type & range)
    { insert(range); }
    iset(const iset & other) = default;
    iset(const parent_type & other) : parent_type(other) {}
    iset(iset && other) = default;
    iset(parent_type && other) noexcept: parent_type(std::move(other)) {}
    explicit iset(value_type value) { insert(value); }

    //! Parse the string.  Use limited matlab format as explained for
    //! operator<< .
    explicit iset(const std::string & text) : parent_type(text) {}
    
    iset & operator=(const parent_type & other)
    { parent_type::operator=(other); return *this; }
    iset & operator=(const iset & other)= default;
    iset & operator=(iset && other)= default;
    iset & operator=(parent_type && other) noexcept
    {
        *static_cast<parent_type*>(this)= std::move(other);
        return *this;
    }

}; // class iset

} // namespace big_model/small_model

using namespace HCSP_SET_MODEL;

} // namespace hcsp

#endif // HCSP_ISET_HPP_
