// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2010, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2010, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#ifndef HCSP_OPERATOR_MISC_TYPES_HPP
#define HCSP_OPERATOR_MISC_TYPES_HPP

namespace hcsp {

/**
 * @brief The way a subexpression should be evaluated.
 *
 * The subexpression may be evaluated exactly or with some bound
 */
enum eval_type {
    //! The constraint type was not recorded yet
    UNKNOWN_EVAL=0,
    
    //! Constrain the lower bound of the subexpression
    LOWER_BOUND_EVAL=1,
    
    //! The subexpression is IMPLIED. It is equivalent with LOWER_BOUND_EVAL
    //! because LOWER_BOUND_EVAL means  subexpression >= external_value.
    //! When external_value==1, then subexpression is forced to 1. If
    //! external_value==0, then subexpression can be either 0 or 1. In other
    //! words external_value --> subexpression.
    IMPLIED_EVAL= LOWER_BOUND_EVAL,
    
    //! Constrain the upper bound of the subexpression
    UPPER_BOUND_EVAL=2,
    
    //! The subexpression implies a value. It is equivalent with
    //! UPPER_BOUND_EVAL because LOWER_BOUND_EVAL means
    //! external_value>=subexpression.  When subexpression==1, then
    //! external_value is forced to 1. If subexpression==0, then external_value
    //! can be either 0 or 1. In other words subexpression --> external_value.
    IMPLIES_EVAL= UPPER_BOUND_EVAL,
    
    //! Constrain the upper and lower bounds of the subexpression,
    //! but don't work harder than that.
    BOTH_BOUNDS_EVAL= (LOWER_BOUND_EVAL| UPPER_BOUND_EVAL),

    //! Constrain the expression exactly to one value.
    STRICT_EVAL= (4 | BOTH_BOUNDS_EVAL)
};

inline eval_type negate_type(eval_type in)
{
    switch (in) {
      case LOWER_BOUND_EVAL: return UPPER_BOUND_EVAL;
      case UPPER_BOUND_EVAL: return LOWER_BOUND_EVAL;
      default:               return in;
    }
}

inline eval_type operator|(eval_type a, eval_type b)
{ return static_cast<eval_type>((int)a | (int)b); }

inline eval_type & operator|=(eval_type & a, eval_type b)
{ return a= (a | b); }


} // namespace hcsp

#endif // HCSP_OPERATOR_MISC_TYPES_HPP
