// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#ifndef HCSP_PROOF_PRINT_VISITOR_HPP_
#define HCSP_PROOF_PRINT_VISITOR_HPP_

#include <proof_visitor.hpp>
#include <iosfwd>

namespace hcsp {

/**
 * @brief A visitor that prints the proof during proof graph traversal
 */
class proof_print_visitor : public proof_visitor
{
  public:
    proof_print_visitor(std::ostream & out,
                        const proof_collector &collector)
        : m_out(out), m_collector(collector) {}
    virtual void visit_explanation(int explanation_clause_id);

    virtual void visit_resolution(
        unsigned pivot,
        int resolvent1_id, const clause_data & resolvent1,
        int resolvent2_id, const clause_data & resolvent2,
        int consequent_id, const clause_data & consequent);
    
    virtual void visit_initial_resolution(
        unsigned variable, const iset & domain,
        int resolvent_id, const clause_data & resolvent,
        int consequent_id, const clause_data & consequent);
  private:
    std::ostream & m_out;
    const proof_collector & m_collector;
}; // class proof_print_visitor

} // namespace hcsp

#endif // HCSP_PROOF_PRINT_VISITOR_HPP_

