// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#ifndef HCSP_PROOF_VISITOR_HPP_
#define HCSP_PROOF_VISITOR_HPP_


#include <iset.hpp>
#include <iosfwd>
#include <proof_collector.hpp>

namespace hcsp {

/**
 * @brief The visitor interface called during proof graph traversal.
 */
class proof_visitor
{
  public:
    virtual ~proof_visitor() {}

    /**
     * @brief Visit the explanation clause which has this ID.
     *
     * @param explanation_clause_id
     */
    virtual void visit_explanation(int explanation_clause_id) = 0;

    /**
     * @brief Visit a binary resolution.
     *
     * Work on this binary resolution which is now being traversed.
     * @param pivot   The resolution variable.
     * @param resolvent1_id,resolvent1 The first resolvent
     * @param resolvent2_id,resolvent2 The second resolvent
     * @param consequent_id,consequent The consequent of the resolution rule
     */
    virtual void visit_resolution(
        unsigned pivot,
        int resolvent1_id, const clause_data & resolvent1,
        int resolvent2_id, const clause_data & resolvent2,
        int consequent_id, const clause_data & consequent) = 0;

    /**
     * @brief Visit a binary resolution with an initial problem domain
     *
     * Work on this binary resolution which is now being traversed.
     * @param pivot   The resolution variable.
     * @param domain  The initial domain of this variable
     * @param resolvent_id,resolvent The resolvent, i.e., the working clause
     * @param consequent_id,consequent The consequent of the resolution rule
     */
    virtual void visit_initial_resolution(
        unsigned variable, const iset & domain,
        int resolvent_id, const clause_data & resolvent,
        int consequent_id, const clause_data & consequent) = 0;
}; // class proof_visitor


} // namespace hcsp

#endif // HCSP_PROOF_VISITOR_HPP_

