// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#ifndef HCSP_SOLVER_TYPES_HPP_
#define HCSP_SOLVER_TYPES_HPP_

#include <stdint.h>
#include <utilities.hpp>
#include <iset.hpp>

namespace hcsp {

struct solver_types
{
    typedef unsigned long long time_type;
    typedef unsigned change_index_t;
    /**
     * @brief A set of labels to be attached to constraints or to variables
     */
    enum label_type {
        NO_LABEL= 0,                         //!< No label in the set
        A=1,                                 //!< Subproblem A
        B=2,                                 //!< Subproblem B
        ALL_LABELS= (uint32_t)~(uint32_t)0   //!< A set of all possible labels
    };

    //! Join two sets of labels, the right set is possibly given as an integer.
    template<class T>
    friend label_type operator|(label_type value, T added)
    { return  static_cast<label_type>((uint32_t)value | added); }
    
    //! Join a sets of labels, possibly given as an integer, into a given set.
    template<class T>
    friend label_type operator|=(label_type & value, T added)
    { return  value= value|added; }
    
    //! Intersect two sets of labels, the right set is possibly given as an
    //! integer.
    template<class T>
    friend label_type operator&(label_type value, T kept)
    { return  static_cast<label_type>((uint32_t)value & kept); }
    
    //! Intersect a sets of labels, possibly given as an integer, into a given
    //! set.
    template<class T>
    friend label_type operator&=(label_type & value, T kept)
    { return  value= value&kept; }
};
} // namespace hcsp

#endif // HCSP_SOLVER_TYPES_HPP_
