// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2012, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2012, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#ifndef HCSP_TIMER_HPP_
#define HCSP_TIMER_HPP_

#include <iosfwd>
#include <string>

namespace hcsp {

/** \brief This class is used to measure time, and print time differences.
 */
class timer
{
  public:
    //! Construct the timer noting the current time.
    timer() : m_impl(NULL) { construct(); }

    //! Create an exact copy
    timer(const timer &other) : m_impl(NULL)
    {
        construct();
        *this= other;
    }
    
    //! Assign an exact copy
    timer & operator=(const timer &other);

    ~timer();

    /** @brief Print the difference in CPU time between timers.
     *
     * @param out           The stream to print to.
     * @param earlier_time  A timer to which compare 'this' to.
     */
    void print_cpu_diff(std::ostream & out, const timer & earlier_time) const;

    //! Like print_cpu_diff, returning a string instead of printing
    std::string cpu_diff_str(const timer & earlier_time) const;
  private:
    class impl;

    //! Construct a timer, taking current time into account.
    void construct();
    
    impl * m_impl;
}; // class hcsp::timer

} // namespace hcsp

#endif // HCSP_TIMER_HPP_
