// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#ifndef HCSP_TUPLE_SET_HPP_
#define HCSP_TUPLE_SET_HPP_

#include <iosfwd>
#include <vector>
#include <iset.hpp>

namespace hcsp {

class tuple_set
{
  public:
    tuple_set() {}
    void insert(const std::vector<std::vector<int> > & base_tuples);
    const std::vector<std::vector<iset> > & tuples() const { return m_tuples; }
    friend std::ostream & operator<<(std::ostream & o, const tuple_set&);
    
  private:
    //! Function for strict ordering of tuples, ignoring a given column
    class order_ignoring_column;
    void convert_to_conflict_set_maybe();
    void insert_tuples_strictly_between(const std::vector<int> & from,
                                        const std::vector<int> & to);
    void merge_tuples();

    void push_back(const std::vector<int> & tuple);

    static bool is_equal_prefix(unsigned elements_to_compare,
                                const std::vector<iset> & set_tuple,
                                const std::vector<int> & element_tuple);
    static bool is_equal_ignoring_column(unsigned column,
                                         const std::vector<iset> & a,
                                         const std::vector<iset> & b);

    void show_self() const;

    std::vector< std::vector<iset> > m_tuples;
    
}; // class tuple_set

} // namespace hcsp


#endif // HCSP_TUPLE_SET_HPP_

