// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */

#include "stateless_clause_propagator.hpp"
using namespace std;
using namespace hcsp;

// virtual
stateless_clause_propagator::result_type
stateless_clause_propagator::propagate( constraint_runtime_info & info)
{
    m_activated_before= true;
    
    unsigned count_satisfied_literals=0;
    unsigned last_satisfied_literal=0; // initialized to avoid warnings.
    for (unsigned i=0 ; i !=m_literals.size() ; ++i) {
        const iset & domain = info.domain(i);
        if (m_literals[i].contains_all_of(domain))
            return VALID;
        if (m_literals[i].contains_any_of(domain)) {
            last_satisfied_literal= i;
            ++count_satisfied_literals;
        }
    }
    if ( count_satisfied_literals == 0 )
        return UNSAT;
    else if ( count_satisfied_literals == 1 ) {
        iset literal= m_literals[last_satisfied_literal];
        literal.intersect(info.domain(last_satisfied_literal));
        info.set_domain(last_satisfied_literal, move(literal));
        return VALID;
    } else {
        return CONSISTENT;
    }
}

// virtual
bool stateless_clause_propagator::conflicts(const constraint_runtime_info &info)
    const
{
    for (unsigned i=0 ; i !=m_literals.size() ; ++i) {
        if (m_literals[i].contains_any_of(info.domain(i)))
            return false;
    }
    return true;
}

// virtual
bool stateless_clause_propagator::constraint_check(
    const constraint_runtime_info & info) const
{
    for (unsigned i=0 ; i != info.arity() ; ++i)
        if (m_literals[i].contains_all_of(info.domain(i)))
            return true;
    return false;
}

// virtual
void stateless_clause_propagator::print(
    ostream &out, const constraint_runtime_info *p_info) const throw()
{
    out << "stateless_clause[";
    const char *prefix= "";
    for (unsigned i=0 ; i != m_literals.size() ; ++i) {
        cout << prefix;
        if (p_info) {
            p_info->print_param(out, i);
            out << ":";
        }
        out << '{' << m_literals[i] << '}';
        prefix= ", ";
    }
    out << "]\n";
}

void stateless_clause_propagator::throw_modifying_clause_at_runtime()
    throw(runtime_error)
{
    throw runtime_error("Modifying the clause at runtime");
}

