// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2009, Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2009, Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#ifndef HCSP_STATELESS_CLAUSE_PROPAGATOR_HPP_
#define HCSP_STATELESS_CLAUSE_PROPAGATOR_HPP_


#include <iset.hpp>
#include <constraint_propagator.hpp>

namespace hcsp {

//! This is a trivial implementation of a clause propagator.
//! It does not maintain or depend on state, and will work exactly the
//! same way regardless of event sequences.
class stateless_clause_propagator : public constraint_propagator
{
  public:
    typedef std::vector<iset> literals_type;
    
    stateless_clause_propagator() : m_activated_before(false) {}
    stateless_clause_propagator(const stateless_clause_propagator& other)
        : constraint_propagator(other),
        m_literals(other.m_literals), m_activated_before(false)
    {
    }
    literals_type & update_literals() throw(std::runtime_error)
    {
        if (m_activated_before)
            throw_modifying_clause_at_runtime();
        return m_literals;
    }
    const literals_type & literals() const { return m_literals; }

    virtual result_type propagate(constraint_runtime_info & info);
    virtual bool conflicts(const constraint_runtime_info & info) const;
    virtual bool constraint_check(const constraint_runtime_info & info) const;
    virtual stateless_clause_propagator* clone() const
    { return new stateless_clause_propagator(*this); }
    virtual void print(std::ostream &out,
                       const constraint_runtime_info *p_info=NULL)const throw();
  private:
    void throw_modifying_clause_at_runtime() throw(std::runtime_error);
    
    literals_type m_literals;
    bool m_activated_before;
}; // class stateless_clause_propagator
} // namespace hcsp
#endif // HCSP_STATELESS_CLAUSE_PROPAGATOR_HPP_
