// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2013,2014 Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2013,2014 Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */
#include <disjoint_propagation_algo.hpp>

#include "propagator_tester.hpp"

#include <neq_propagators.hpp>
#include <interval_propagators.hpp>
#include <comparison_propagators.hpp>
#include <clause_propagator.hpp>
#include "test_utilities.hpp"

using namespace std;
using namespace hcsp;
using namespace hcsp::random;
using namespace hcsp::test;

namespace {
vector<iset> random_literals()
{
    vector<iset> ret;
    
    if (gen(1000)==0)
        return ret;
    ret.push_back(random_var());
    if (gen(100) == 0)
        return ret;
    generate_n(back_inserter(ret), range(1,7), var_randomizer());
    return ret;
}
class greater_propagator : public difference_ge_propagator {
  public:
    greater_propagator() : difference_ge_propagator(0) {}
    greater_propagator *clone() const { return new greater_propagator(*this); }
};

} // namespace

void test_multi_disjoint_propagator()
{
    multi_disjoint_propagator propagator;
    if (gen(1000) == 0) {
        propagator.update_literals()= random_literals();
        propagator_tester<multi_disjoint_propagator>(propagator.count_params(),
                                                     propagator).run();
        return;
    }
    unique_ptr<constraint_propagator> disjunct;
    unsigned disjunct_count= hcsp::random::range(1, 3);
    for (unsigned i=0 ; i != disjunct_count ; ++i) {
        switch (gen(4)) {
          case 0:
            disjunct.reset(
                new sub_interval_propagator(
                    random_var_maybe_out_of_bound(iset::MIN_DOMAIN/2), 
                    random_var_maybe_out_of_bound(iset::MAX_DOMAIN/2)));
            break;
          case 1:  disjunct.reset(new neq_propagator);   break;
          case 2:  disjunct.reset(new equal_propagator); break;
          case 3:  disjunct.reset(new greater_propagator); break;
          default: {
              clause_propagator *p_clause= new clause_propagator;
              disjunct.reset(p_clause);
              p_clause->update_literals()= random_literals();
          }
        }
        unsigned params = disjunct->count_params();
        propagator.append_constraint(move(disjunct), params);
    }
    if (gen(100) == 0)
        propagator.update_literals()= random_literals();
    propagator_tester<multi_disjoint_propagator>(propagator.count_params(),
                                                 propagator).run();
}

int main(int argc, char *argv[])
{
    HCSP_PROFILE_CURRENT;
    BEGIN_TEST_MAIN;
    cout << "Seed is " << set_seed(argc, argv) << endl;
     for (unsigned i=0 ; i < 3000 ; ++i)
         test_multi_disjoint_propagator();
    for (unsigned i=0 ; i < 1000 ; ++i) {
        //
        // Testing !([x,y] subset_of [a,b]) || u > v
        //
        typedef disjoint_propagation_algo<
            sub_interval_propagator,
            greater_propagator> sub_interval_or_greater_algo;
        typedef constraint_propagator_from_algo<sub_interval_or_greater_algo>
            sub_interval_or_greater_propagator;
        propagator_tester<sub_interval_or_greater_propagator>(
            4, random_var_maybe_out_of_bound(iset::MIN_DOMAIN/2), 
            random_var_maybe_out_of_bound(iset::MAX_DOMAIN/2)).run();
        
        //
        // Testing (x1 in lit1 || x2 in lit2 || ... || x_n in lit_n) || u > v
        //
        typedef disjoint_propagation_algo<
            clause_propagation_algo,
            greater_propagator> clause_or_greater_algo;
        typedef constraint_propagator_from_algo<clause_or_greater_algo>
            clause_or_greater_propagator;

        clause_or_greater_propagator propagator;
        clause_propagation_algo::literals_type literals= random_literals();
        propagator.algo().algo_a().update_literals()= literals;
        unsigned arity= literals.size()+2;

        propagator_tester<clause_or_greater_propagator>(arity,
                                                        propagator).run();

        //
        // Testing u > v || (x1 in lit1 || x2 in lit2 || ... || x_n in lit_n)
        //
        typedef disjoint_propagation_algo<
            greater_propagator,
            clause_propagation_algo> greater_or_clause_algo;
        typedef constraint_propagator_from_algo<greater_or_clause_algo>
            greater_or_clause_propagator;

        greater_or_clause_propagator propagator2;
        propagator2.algo().algo_b().update_literals()= literals;
        propagator_tester<greater_or_clause_propagator>(arity,
                                                        propagator2).run();
    }
    END_TEST_MAIN;
}
