// -*- c-basic-offset:4 -*-
/*  This file is part of HaifaCSP Constraint Solver.
 *  Copyright (C) 2014 Michael Veksler mickey.veksler@gmail.com
 *  Copyright (C) 2014 Technion Israel Institute of Technology
 * 
 *  Permission is granted to copy and use the source code, the library, and 
 *  the executable only for academic and evaluation purposes. For any other
 *  use, most notably commercial, you must request permission from the copyright
 *  holders.
 *  Any copy of derivative work must retain this copyright notice.
 */

#include "propagator_tester.hpp"

#include <arithmetic_propagators.hpp>
#include <comparison_propagators.hpp>
#include <interval_propagators.hpp>
#include <table_propagators.hpp>

#include "test_utilities.hpp"

using namespace std;
using namespace hcsp;
using namespace hcsp::random;
using namespace hcsp::test;

int main(int argc, char *argv[])
{
    BEGIN_TEST_MAIN;
    unsigned seed = set_seed(argc, argv);
    cout << "Seed is " << seed << endl;
    for (unsigned i=0 ; i < 3000 ; ++i) {
        if (i != 0) {
            seed = gen();
            set_seed(seed);
        }
        typedef linear_combination_ge_propagator::coefficient_type
                                                             coefficient_type;
        int n_bits= hcsp::random::range(10U,sizeof(coefficient_type)*8);
        int32_t max_for_bits= (1U << n_bits)-1;
        iset::value_range_type bound= random_var_bounds(
            gen(2), -1-max_for_bits, max_for_bits);
        iset::value_type k= random_var_in_bounds(bound.first, bound.second);
        std::vector<coefficient_type> coefficients;
        try {
            while (coefficients.size() < 5) {
                coefficients.push_back(
                    random_var_in_bounds(bound.first, bound.second));
                if (gen(3) == 0)
                    propagator_tester<linear_combination_ge_propagator>(
                        coefficients.size(), k, coefficients).run();
            }
        } catch (...) {
            cout << "Seed is " << seed << endl;
            throw;
        }
    }
    return 0;
    END_TEST_MAIN;
}
